# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime
from lusid.models.post_close_activity import PostCloseActivity

class PostCloseActivitiesRequest(BaseModel):
    """
    PostCloseActivitiesRequest
    """
    post_close_activities: List[PostCloseActivity] = Field(description="A collection of post-close activities.", alias="postCloseActivities")
    __properties = ["postCloseActivities"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PostCloseActivitiesRequest:
        """Create an instance of PostCloseActivitiesRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in post_close_activities (list)
        _items = []
        if self.post_close_activities:
            for _item in self.post_close_activities:
                if _item:
                    _items.append(_item.to_dict())
            _dict['postCloseActivities'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PostCloseActivitiesRequest:
        """Create an instance of PostCloseActivitiesRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PostCloseActivitiesRequest.parse_obj(obj)

        _obj = PostCloseActivitiesRequest.parse_obj({
            "post_close_activities": [PostCloseActivity.from_dict(_item) for _item in obj.get("postCloseActivities")] if obj.get("postCloseActivities") is not None else None
        })
        return _obj

PostCloseActivitiesRequest.update_forward_refs()
